import sys
import os
import re

import pathlib

import datetime, time

from PyQt5 import QtWidgets, QtGui, QtCore    #, uic, QtQml, QtQuick
from PyQt5.QtWidgets import QFileDialog, QMessageBox
from PyQt5.QtCore import Qt, QTimer
from PyQt5.QtGui import QColor

import json

import GraphFrame, GroupsFrame, LabelFrame, iws_defs

#from HkMonitorTabWidget import TabBar

# floating point number match pattern
fpn = re.compile('[+-]?(\d+(\.\d*)?|\.\d+)([eE][+-]?\d+)?')


class HkMonitorUi(QtWidgets.QWidget):

    def __init__(self, parent):
        super(HkMonitorUi, self).__init__()

        self.parent = parent

        #self.parameters = {}
        #self.graph_parameters = {}
        self.groups_frames = []
        self.graph_frames = []

        self.windows = {}
        self.plot_windows = {}

        self.all_labels = {}
        self.all_plot_labels = {}
        self.all_mplot_labels = {}

        self.all_labels_for_update = {}

        self.downsample = 5
        self.downcounter = self.downsample

        self.top_frame_parameters = {}

        
    def layout(self, parent, config_file, config_type, main):

        self.config_file = config_file

        with open(config_file) as f:
            data = json.load(f)

        # Not used
        #box_frame = self.make_box_frame()

        self.top_frame = self.make_top_frame(main)
        self.middle_frame = self.make_middle_frame(data)

        self.bottom_frame = self.make_bottom_frame()

        layout = QtWidgets.QVBoxLayout()
        layout.addWidget(self.top_frame, stretch = 0)
        layout.addWidget(self.middle_frame, stretch = 1)

        layout.addWidget(self.bottom_frame)

        statusbar = QtWidgets.QStatusBar(parent)
        statusbar.setObjectName("statusbar")
        statusbar.showMessage("Using configuration: " + config_file)

        self.ini_textbox = QtWidgets.QLabel()
        self.ini_textbox.setText("Ini Type: " + config_type)

        self.file_textbox = QtWidgets.QLabel()

        self.spare_textbox = QtWidgets.QLabel()
        self.spare_textbox.setText('                    ')
        self.obt_textbox = QtWidgets.QLabel()
        self.obt_textbox.setText('                    ')
        #self.tm_packet_counter_textbox = QtWidgets.QLabel()
        #self.tm_packet_counter_textbox.setText('TM Packets:        0 ')

        eof_label = QtWidgets.QLabel('EOF:')
        self.eof_led = QtWidgets.QPushButton()
        #self.eof_led.setStyleSheet("background-color: green")
        style_sheet = "QPushButton[stat='True'] {background-color: green} QPushButton[stat='False'] {background-color:red}"
        #style_sheet = "background-color: green"
        self.eof_led.property = 'True'
        self.eof_led.setProperty('stat', 'True')
        self.eof_led.setStyleSheet(style_sheet)
        self.eof_led.setFixedSize(10,10)

        statusbar.reformat()
        statusbar.addPermanentWidget(self.ini_textbox)
        statusbar.addPermanentWidget(self.file_textbox)
        statusbar.addPermanentWidget(self.spare_textbox)
        statusbar.addPermanentWidget(self.obt_textbox)
        #statusbar.addPermanentWidget(self.tm_packet_counter_textbox)
        statusbar.addPermanentWidget(eof_label)
        statusbar.addPermanentWidget(self.eof_led)
        #statusbar.addPermanentWidget(self.spare_textbox)

        parent.setStatusBar(statusbar)

        self.setLayout(layout)

        #self.debug()
        #if 'VOGT2856' in self.all_labels:
            #print(f"{self.all_labels['VOGT2856']}")

        # Create master dictionary of all the parameters - only use if not telling each form to update
        #for frame in self.groups_frames:
            #self.parameters.update(frame.parameters())

        #for frame in self.graph_frames:
            #self.graph_parameters.update(frame.parameters())

        #print("End of Ui init")


    def debug(self):
        #print("GroupFrames:")
        #for frame in self.groups_frames:
            #frame.debug()
        #print()

        #for label in self.parameters:
            #print(label)

        #print("Graph labels:")
        #for frame in self.graph_frames:
            #frame.debug()
        #print()

        #for label in self.graph_parameters:
            #print(label)

        # {'VDGT2771': <GroupsFrame.GroupsFrame object at 0x7fbbe281ae58>, 'VDGT2772': <GroupsFrame.GroupsFrame object at 0x7fbbe281ae58>, ... }
        print('all_labels:')
        #print(f'{self.all_labels}')
        for k,v in self.all_labels.items():
            v.debug()
            #print(f'f = {f}')
        print()

        #print('all_plot_labels:')
        #print(f'{self.all_plot_labels}')
        #print()

        
    def make_top_left_frame(self, main):
        '''Create a frame to contain widgets appearing on the left hand side of the top frame.

        Parameters
        ----------
        main : Bool
               Controls something or rather.

        Returns
        -------
        A frame containing widgets.
        '''
        frame = QtWidgets.QFrame()
        frame.setFrameStyle(QtWidgets.QFrame.StyledPanel)
        layout = QtWidgets.QHBoxLayout()

        mode_label = QtWidgets.QLabel()
        mode_label.setText('Operating Mode:')
        self.hk_monitor_mode = QtWidgets.QLabel()
        self.hk_monitor_mode.setText('Idle')

        layout.addWidget(mode_label)
        layout.addWidget(self.hk_monitor_mode)

        if main:
            self.start_button = QtWidgets.QPushButton("Start")
            self.start_button.setEnabled(True)
            self.start_button.setAccessibleName("START")
            layout.addWidget(self.start_button)

        #layout.setContentsMargins(5, 0, 0, 0)
        frame.setLayout(layout)
        return frame


    def make_top_middle_frame(self):
        '''Create a frame to contain widgets appearing in the middle part of the top frame.

        Parameters
        ----------
        none

        Returns
        -------
        A frame containing widgets.
        '''
        frame = QtWidgets.QFrame()
        frame.setFrameStyle(QtWidgets.QFrame.StyledPanel)
        layout = QtWidgets.QHBoxLayout()

        font = self.font()
        font.setPointSize(16)
        science_mode_label = QtWidgets.QLabel('Science Mode:')
        science_mode_label.setToolTip('VOGT2856')
        self.science_mode  = QtWidgets.QLabel('None')
        science_mode_label.setFont(font)
        self.science_mode.setFont(font)

        #self.top_frame_parameters['VOGT2856'] = (science_mode_label, self.science_mode)
        self.top_frame_parameters['VOGT2856'] = self.science_mode

        layout.addStretch(1)
        layout.addWidget(science_mode_label)
        layout.addWidget(self.science_mode)
        layout.addStretch(1)

        frame.setLayout(layout)
        return frame


    def make_top_right_frame(self):
        '''Create a frame to contain widgets appearing on the right hand side part of the top frame.

        Parameters
        ----------
        none

        Returns
        -------
        A frame containing widgets.
        '''
        frame = QtWidgets.QFrame()
        frame.setFrameStyle(QtWidgets.QFrame.StyledPanel)

        layout = QtWidgets.QHBoxLayout()

        self.new_config_button             = QtWidgets.QPushButton("New Config")

        combobox = QtWidgets.QComboBox()
        fname = os.path.splitext(os.path.basename(self.config_file))[0]
        combobox.addItem(fname)
        cdir = pathlib.Path(iws_defs.defaults_directory)
        for f in cdir.glob('*json'):
            if f.stem != fname:
                combobox.addItem(f.stem)
        self.config_combo_box = combobox

        self.new_plot_window_button        = QtWidgets.QPushButton("New Plot")

        self.multi_plot_window_combobox    = QtWidgets.QComboBox()
        self.multi_plot_window_combobox.addItem('MultiPlot')
        mp_dir = pathlib.Path(iws_defs.multi_plots_directory)
        for f in mp_dir.glob('*json'):
            self.multi_plot_window_combobox.addItem(f.stem)

        self.open_window_button            = QtWidgets.QPushButton("Open Window")
        self.open_plot_window_button       = QtWidgets.QPushButton("Open Plot")
        self.open_multi_plot_window_button = QtWidgets.QPushButton("Open Multi Plot")
        self.quit_button                   = QtWidgets.QPushButton("Quit")

        #layout.addWidget(self.new_config_button)
        layout.addWidget(self.config_combo_box)
        #layout.addWidget(self.new_plot_window_button)
        layout.addWidget(self.open_window_button)
        layout.addWidget(self.open_plot_window_button)
        #layout.addWidget(self.open_multi_plot_window_button)
        layout.addWidget(self.multi_plot_window_combobox)
        layout.addWidget(self.quit_button)

        frame.setLayout(layout)
        return frame


    # This frame contains the various controls and state indicators
    def make_top_frame(self, main):
        '''Create a frame to contain sub-frames along the top of the window.

        Parameters
        ----------
        main : boolean

        Returns
        -------
        A frame containing widgets.
        '''
        frame = QtWidgets.QFrame()
        #frame.setStyleSheet("background-color:yellow;") 
        #frame.setFrameStyle(QtWidgets.QFrame.StyledPanel)

        layout = QtWidgets.QHBoxLayout()

        layout.addWidget(self.make_top_left_frame(main))
        layout.addWidget(self.make_top_middle_frame())
        layout.addWidget(self.make_top_right_frame())
        layout.setContentsMargins(0, 0, 0, 0)
        frame.setLayout(layout)
        return frame


    # Contains the frames containing the packet/event text displays, the parameter displays and the plots
    def make_middle_frame(self, json_data):
        frame = QtWidgets.QFrame()
        #frame.setStyleSheet("background-color:blue;") 

        layout = QtWidgets.QHBoxLayout()

        left_frame  = self.make_left_frame()
        right_frame = self.make_right_frame(json_data)

        layout.addWidget(left_frame)
        layout.addWidget(right_frame)
        frame.setLayout(layout)
        return frame


    def make_packets_box(self, font):
        box = QtWidgets.QGroupBox("TC/TM Packets")

        self.event_text  = QtWidgets.QTextEdit()
        #self.event_text.setStyleSheet('color: green;')

        self.event_text.setFont(font)
        self.event_text.setReadOnly(True)

        self.event_clear_button = QtWidgets.QPushButton('Clear')
        self.event_clear_button.clicked.connect(self.event_clear_button_pressed)

        self.unknown_packet_counter_textbox = QtWidgets.QLabel()
        self.unknown_packet_counter_textbox.setText('Unknown: 0')
        self.unknown_packet_counter_textbox.setFrameStyle(QtWidgets.QFrame.Panel | QtWidgets.QFrame.Sunken)

        self.tc_packet_counter_textbox = QtWidgets.QLabel()
        self.tc_packet_counter_textbox.setText('TC Packets: 0')
        self.tc_packet_counter_textbox.setFrameStyle(QtWidgets.QFrame.Panel | QtWidgets.QFrame.Sunken)

        self.tm_packet_counter_textbox = QtWidgets.QLabel()
        self.tm_packet_counter_textbox.setText('TM Packets: 0')
        self.tm_packet_counter_textbox.setFrameStyle(QtWidgets.QFrame.Panel | QtWidgets.QFrame.Sunken)

        frame = QtWidgets.QFrame()
        layout = QtWidgets.QHBoxLayout()

        layout.addWidget(self.event_clear_button)
        layout.addStretch(1)
        layout.addWidget(self.tc_packet_counter_textbox)
        layout.addWidget(self.tm_packet_counter_textbox)
        layout.addWidget(self.unknown_packet_counter_textbox)
        frame.setLayout(layout)

        event_layout = QtWidgets.QVBoxLayout()
        event_layout.setContentsMargins(0, 0, 0, 0)
        event_layout.addWidget(self.event_text)
        event_layout.addWidget(frame)
        box.setLayout(event_layout)

        return box


    def make_packet_counter_frame(self):
        frame  = QtWidgets.QFrame()
        layout = QtWidgets.QHBoxLayout()

        self.rt_fast_packet_counter_textbox  = QtWidgets.QLabel('rt fast:         0')
        self.rt_slow_packet_counter_textbox  = QtWidgets.QLabel('rt slow:         0')
        self.rt_sfast_packet_counter_textbox = QtWidgets.QLabel('rt superfast:        0')
        self.st_slow_packet_counter_textbox  = QtWidgets.QLabel('st slow:         0')
        self.st_sfast_packet_counter_textbox = QtWidgets.QLabel('st superfast:        0')

        self.hk_packet_counter_textbox = QtWidgets.QLabel()
        self.hk_packet_counter_textbox.setText('Total HK Packets: 0')
        self.hk_packet_counter_textbox.setFrameStyle(QtWidgets.QFrame.Panel | QtWidgets.QFrame.Sunken)

        self.rt_fast_packet_counter_textbox.setFrameStyle(QtWidgets.QFrame.Panel | QtWidgets.QFrame.Sunken)
        self.rt_slow_packet_counter_textbox.setFrameStyle(QtWidgets.QFrame.Panel | QtWidgets.QFrame.Sunken)
        self.rt_sfast_packet_counter_textbox.setFrameStyle(QtWidgets.QFrame.Panel | QtWidgets.QFrame.Sunken)
        self.st_slow_packet_counter_textbox.setFrameStyle(QtWidgets.QFrame.Panel | QtWidgets.QFrame.Sunken)
        self.st_sfast_packet_counter_textbox.setFrameStyle(QtWidgets.QFrame.Panel | QtWidgets.QFrame.Sunken)

        label = QtWidgets.QLabel('Downsample')
        self.downcount_spinbox = QtWidgets.QSpinBox(self)
        self.downcount_spinbox.setValue(self.downsample)
        self.downcount_spinbox.valueChanged.connect(self.downsample_spinbox_value_changed)
#        self.downcount_spinbox.setLabel('Downcount')

        layout.addWidget(self.rt_fast_packet_counter_textbox)
        layout.addWidget(self.rt_slow_packet_counter_textbox)
        layout.addWidget(self.rt_sfast_packet_counter_textbox)
        layout.addWidget(self.st_slow_packet_counter_textbox)
        layout.addWidget(self.st_sfast_packet_counter_textbox)
        layout.addWidget(self.hk_packet_counter_textbox)
        layout.addStretch(1)
        layout.addWidget(label)
        layout.addWidget(self.downcount_spinbox)

        frame.setLayout(layout)
        return frame


    def downsample_spinbox_value_changed(self, val):
        print(f'setting downsample to {val}')
        self.downsample = val
        pass


    def make_events_box(self, font):
        box = QtWidgets.QGroupBox("Events")

        self.packet_text = QtWidgets.QPlainTextEdit()
        #self.packet_text.setStyleSheet('color: green;')

        self.packet_text.setFont(font)
        self.packet_text.setReadOnly(True)

        self.packet_clear_button = QtWidgets.QPushButton('Clear')
        self.packet_clear_button.clicked.connect(self.packet_clear_button_pressed)

        self.event_counter_textbox = QtWidgets.QLabel()
        self.event_counter_textbox.setText('Events: 0')
        self.event_counter_textbox.setFrameStyle(QtWidgets.QFrame.Panel | QtWidgets.QFrame.Sunken)

        #self.hk_packet_counter_textbox = QtWidgets.QLabel()
        #self.hk_packet_counter_textbox.setText('Hk Packets: 0')
        #self.hk_packet_counter_textbox.setFrameStyle(QtWidgets.QFrame.Panel | QtWidgets.QFrame.Sunken)

        frame = QtWidgets.QFrame()
        layout = QtWidgets.QHBoxLayout()

        new = False
        if new:
            # New
            container_frame  = QtWidgets.QFrame()
            container_layout = QtWidgets.QVBoxLayout()
            container_layout.setContentsMargins(0, 0, 0, 0)

            counter_frame = self.make_packet_counter_frame()

            total_counter_frame  = QtWidgets.QFrame()
            total_counter_layout = QtWidgets.QHBoxLayout()
            total_counter_layout.setContentsMargins(0, 0, 0, 0)

            total_counter_layout.addWidget(self.packet_clear_button)
            total_counter_layout.addStretch(1)
            total_counter_layout.addWidget(self.event_counter_textbox)
            #total_counter_layout.addWidget(self.hk_packet_counter_textbox)

            total_counter_frame.setLayout(total_counter_layout)
            
            container_layout.addWidget(counter_frame)
            container_layout.addWidget(total_counter_frame)
            
            container_frame.setLayout(container_layout)
            
            layout.addWidget(container_frame)
            
            frame.setLayout(layout)
            
            # End of new
        else:
            layout.addWidget(self.packet_clear_button)
            layout.addStretch(1)
            layout.addWidget(self.event_counter_textbox)
            #layout.addWidget(self.hk_packet_counter_textbox)
            #layout.addWidget(counter_frame)
            frame.setLayout(layout)

        packet_layout = QtWidgets.QVBoxLayout()
        packet_layout.setContentsMargins(0, 0, 0, 0)
        packet_layout.addWidget(self.packet_text)
        packet_layout.addWidget(frame)

        box.setLayout(packet_layout)

        return box


    # Contains the event and packet text views
    def make_left_frame(self):
        #font = QtGui.QFont()
        #font.setPointSize(6)
        font = QtGui.QFont("Consolas", 6)
        ###font = QtGui.QFont("Deja Vu Sans Mono", 6)
        ###font = QtGui.QFont("Liberation Mono", 6)

        frame = QtWidgets.QFrame()
        #frame.setMinimumSize(QtCore.QSize(iws_defs.TEXT_FIELD_MINIMUM_SIZE, 170))
        frame.setMinimumSize(QtCore.QSize(iws_defs.TEXT_FIELD_MINIMUM_SIZE, 185))
        #frame.setStyleSheet("background-color:green")

        layout = QtWidgets.QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)

        packet_box  = self.make_packets_box(font)
        event_box = self.make_events_box(font)

#        layout.addWidget(event_box)
        layout.addWidget(packet_box)

        frame.setLayout(layout)
        return frame


    # This contains the plot and parameter views
    def make_right_frame(self, json_data):
        frame = QtWidgets.QFrame()
        #frame.setStyleSheet("background-color:red")
        #upper_frame = self.make_upper_frame(json_data['graphs'])
        lower_frame = self.make_lower_frame(json_data['tabs'])

        layout = QtWidgets.QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)

        #layout.addWidget(upper_frame)
        layout.addWidget(lower_frame)
        frame.setLayout(layout)
        return frame


    def remake_upper_frame(self, json_data):
        return
        """
        right_frame = self.middle_frame.layout().itemAt(1).widget()
        upper_frame = right_frame.layout().itemAt(0).widget()
        #tab_frame   = lower_frame.layout().itemAt(0).widget()

        tab_widget  = upper_frame.layout().itemAt(0).widget()
        print(f"tab_widget in remake is {tab_widget}")

        for i in json_data:
            tab_name = i['name']
            gtab = GraphFrame.GraphFrame(tab_name, i)
            #gtab.setStyleSheet("background-color:yellow")
            tab_widget.addTab(gtab, tab_name)
            self.graph_frames.append(gtab)

        """

    """
    json = [{"name":n,"naxes":n,"yaxis1":{},"yaxis2:{},"labels:[]}]
    """
    # This is the frame containing the plot view
    def make_upper_frame(self, json_data):
        tab_frame = QtWidgets.QFrame()
        #tab_frame.setStyleSheet("background-color:orange;") 

        tab_widget = QtWidgets.QTabWidget()
        #tab_widget.setStyleSheet("background-color:purple;") 

        font = self.font()
        font.setBold(True)
        tab_widget.setFont(font)

        for i in json_data:
            tab_name = i['name']
            gtab = GraphFrame.GraphFrame(tab_name, i, self)
            #gtab.setStyleSheet("background-color:yellow")
            tab_widget.addTab(gtab, tab_name)
            self.graph_frames.append(gtab)

        layout = QtWidgets.QHBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)

        layout.addWidget(tab_widget)
        tab_frame.setLayout(layout)
        #return tab_widget
        return tab_frame


    def remake_lower_frame(self, json_data):
        right_frame = self.middle_frame.layout().itemAt(1).widget()
        lower_frame = right_frame.layout().itemAt(1).widget()
        #tab_frame   = lower_frame.layout().itemAt(0).widget()

        tab_widget  = lower_frame.layout().itemAt(0).widget()
        print(f"tab_widget in remake is {tab_widget}")

        n = 0
        for i in json_data:
            tab_name = i['name']
            ltab = GroupsFrame.GroupsFrame(i['groups'], n, tab_widget, self)
            ltab.group_name = tab_name
            #ltab.setStyleSheet("background-color:yellow")
            tab_widget.addTab(ltab, tab_name)
            n += 1
            self.groups_frames.append(ltab)


    """ [{'name': 'rt_fast', 'groups': [{'row': 0, 'column': 0, 'labels': [{'mib_name': 'VIS_ROE_VCCD_01', 'mib_id': 'VS1T2038', 'source': 'rt_fast', 'unit': 'V'}, { """
    # This is the parameters view
    def make_lower_frame(self, json_data):
        tab_frame = QtWidgets.QFrame()
        #tab_frame.setStyleSheet("background-color:orange;") 

        tab_widget = QtWidgets.QTabWidget()
        font = self.font()
        font.setBold(True)
        tab_widget.setFont(font)

        #tab_widget.setTabBar(TabBar(self))

        #tab_widget.setStyleSheet("background-color:purple;") 

        #p = self.tabWidget.palette()
        #p.setColor(self.tabWidget.backgroundRole(), Qt.red)
        #self.tabWidget.setPalette(p)

        #tab_widget.setStyleSheet('QTabBar::tab {background-color: red;}')
    
        n = 0
        for i in json_data:
            tab_name = i['name']
            ltab = GroupsFrame.GroupsFrame(i['groups'], n, tab_widget, self)
            ltab.group_name = tab_name
            tab_widget.addTab(ltab, tab_name)
            n += 1
            self.groups_frames.append(ltab)

        layout = QtWidgets.QHBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(tab_widget)
        tab_frame.setLayout(layout)
        #return tab_widget
        return tab_frame

    def make_bottom_frame(self):
        #frame = QtWidgets.QFrame()
        ##frame.setStyleSheet("background-color:pink")

        #layout = QtWidgets.QHBoxLayout()
        #layout.setContentsMargins(0, 0, 0, 0)
        return self.make_packet_counter_frame()


    def event_clear_button_pressed(self):
        self.event_text.clear()
        pass

        
    def packet_clear_button_pressed(self):
        self.packet_text.clear()
        pass


    def clear_plotdata(self):
        for frame in self.graph_frames:
            frame.clear_plot_data()


    def update_tab_colour(self):
        for frame in self.groups_frames:
            frame.update_tab_colour()


    def update_tm(self, parameter_string):
        mib_id = parameter_string[iws_defs.TM_MIB_ID_INDEX]
        
        #print(f'MIB_ID = {mib_id}')
        #if mib_id == 'VOGT2856':
            #print('mib id = ' + mib_id)
                

        try:
            if mib_id in self.all_labels:
                self.all_labels[mib_id].update_tm(parameter_string, mib_id)
                #(mib_label, _) = self.all_labels[mib_id]
                #mib_label.update_tm(parameter_string)
                ##self.all_labels[mib_id].update_tm(parameter_string)

            if mib_id in self.top_frame_parameters:
                s = self.all_labels[mib_id].all_labels[mib_id].eng_string
                #print(f'Setting top parameter {mib_id} to \'{s}\'')
                #self.top_frame_parameters[mib_id].setText(self.all_labels[mib_id].eng_string)
                self.top_frame_parameters[mib_id].setText(s)

        except Exception as e:
            print(f'Exception: {e} in HkMonitorUi:update_tm')
            #if mib_id == 'VOGT2856':
                ##print(f'[{mib_id}] caused exception in update_tm in HkMonitorUi all_labels')
                #pass
            pass


        #try:
            #if mib_id in self.all_plot_labels:
                #self.all_plot_labels[mib_id].update_tm(parameter_string)
        #except Exception as e:
            #if mib_id == 'VOGT2856':
                ##print(f'[{mib_id}] caused exception in update_tm in HkMonitorUi all_plot_labels')
                #pass
            #pass

        #for frame in self.groups_frames:
            ##frame.update_tm(parameter_string)
            #if frame.update_tm(parameter_string):
                #break

        #for frame in self.graph_frames:
            #frame.update_tm(parameter_string)

        for window in self.windows.values():
            window.update_tm(parameter_string)

        #try:
            #if mib_id in self.all_mplot_labels:
                ##print(f'{mib_id} in all_mplot_labels')
                #self.all_mplot_labels[mib_id].update_tm(parameter_string, display=False)
        #except Exception as e:
            ##if mib_id == 'VOGT2856':
                ###print(f'[{mib_id}] caused exception in update_tm in HkMonitorUi all_plot_labels')
                ##pass
            #pass

        for window in self.plot_windows.values():
            window.update_tm(parameter_string)


    def update_plots(self, obt_time, time_conversion_string):
        #for frame in self.groups_frames:
            #frame.update_tab_colour()

        #for window in self.windows.values():
            #window.update_tab_colour()

        #for frame in self.graph_frames:
            #frame.update_plot(obt_time, time_conversion_string)

        if self.downcounter == 0:
            self.downcounter = self.downsample

            get_time = datetime.datetime.strptime(obt_time, time_conversion_string)
            xaxis_time_str = str(get_time.hour) + ":" + str(get_time.minute) + ":" + str(get_time.second)
            xval = int(time.mktime(get_time.timetuple()))

            for window in self.plot_windows.values():
                window.update_plot(obt_time, time_conversion_string, xval)
        else:
            self.downcounter -= 1

        '''
        if self.downcounter >= self.downsample:
            self.downcounter = 0

            get_time = datetime.datetime.strptime(obt_time, time_conversion_string)
            xaxis_time_str = str(get_time.hour) + ":" + str(get_time.minute) + ":" + str(get_time.second)
            xval = int(time.mktime(get_time.timetuple()))

            for window in self.plot_windows.values():
                window.update_plot(obt_time, time_conversion_string, xval)
        else:
            self.downcounter += 1
        '''

    def child_window_closed(self, child):
        print(f'Closing child window {child}')
        self.parent.child_window_closed(child)
